<?php
/**
 * Created by PhpStorm.
 * User: FrédéricGuichard
 * Date: 28/09/2016
 * Time: 14:58
 */

namespace ByDedi\AdminBundle\EventListener;

use Knp\Menu\MenuFactory;
use Knp\Menu\MenuItem;
use Sonata\AdminBundle\Admin\AbstractAdmin;
use Sonata\AdminBundle\Event\ConfigureMenuEvent;
use Symfony\Component\DependencyInjection\ContainerInterface;

class MenuBuilderListener
{
    private $container;

    public function __construct(ContainerInterface $container)
    {
        $this->container = $container;
    }

    public function addMenuItems(ConfigureMenuEvent $event)
    {
        $user = $this->container->get('security.token_storage')->getToken()->getUser();
        $translator = $this->container->get('translator');

        $em = $this->container->get('doctrine.orm.entity_manager');
        $request = $this->container->get('request_stack')->getCurrentRequest();
        $userBmRepo = $em->getRepository('AdminBundle:UserBookmark');

        $bookmarks = $userBmRepo->findBy(['user' => $user]);

        $menu = $event->getMenu();

        if (!empty($bookmarks)) {
            $children = $menu->getChildren();

            $newMenuOrder = array_keys($children);

            $bookmarkMenuLabel = 'Favoris';
            array_unshift($newMenuOrder, $bookmarkMenuLabel);

            $options = ['icon' => 'fa fa-star', 'showSpanTag' => false];
            $favorisMenu = new MenuItem($bookmarkMenuLabel, new MenuFactory());
            $favorisMenu->setExtras($options);
            $child = $menu->addChild($favorisMenu);

            foreach ($bookmarks as $b) {
                $favorisItem = new MenuItem($b->getTitle(), new MenuFactory());
                $favorisItem->setUri($b->getUrl());
                $child->addChild($favorisItem);
            }

            $menu->reorderChildren($newMenuOrder);
        }

        $children = $menu->getChildren();

        foreach ($children as $name => $child) {
            $child->setExtra('showSpanTag', true);
            $subChild = $child->getChildren();
            if (!empty($subChild)) {
                foreach ($subChild as $sName => $sChild) {
                    $sChild->setExtra('showSpanTag', false);
                    $extras = $sChild->getExtras();
                    if (!empty($extras['admin'])) {
                        $adminClass = $extras['admin'];
                        /**
                         * @var $adminClass AbstractAdmin
                         */
                        $parentMenu = $adminClass->getParentMenu();
                        if (!empty($parentMenu)) {
                            $parentMenuName = $translator->trans(key($parentMenu), [], $adminClass->getTranslationDomain());
                            $parentMenuItem = $child->getChild($parentMenuName);
                            if ($parentMenuItem == null) {
                                $parentMenuItem = $child->addChild($parentMenuName, ['extras' => ['icon' => 'fa fa-angle-right', 'showSpanTag' => false]]);
                            }
                            $copyItem = $sChild->copy();
                            $child->removeChild($sChild);
                            $parentMenuItem->addChild($copyItem)->actsLikeLast();
                        }
                    }
                }
            }
        }

        $items = $menu->getChildren();
        $requestUri = $request->getRequestUri();
        foreach ($items as $item) {
            if (str_replace($request->getScheme().'://'.$request->getHost(), '', $item->getUri()) === $requestUri) {
                $item->setCurrent(true);
            }
            $subChild = $item->getChildren();
            foreach ($subChild as $sitem) {
                if (str_replace($request->getScheme().'://'.$request->getHost(), '', $sitem->getUri()) === $requestUri) {
                    $sitem->setCurrent(true);
                    $item->setCurrent(true);
                }
                $subSubChild = $sitem->getChildren();
                foreach ($subSubChild as $ssitem) {
                    if (str_replace($request->getScheme().'://'.$request->getHost(), '', $ssitem->getUri()) === $requestUri) {
                        $ssitem->setCurrent(true);
                        $sitem->setCurrent(true);
                        $item->setCurrent(true);
                    }
                }
            }
        }
    }
}
